require_relative 'player'
require_relative 'die'
require_relative 'game_turn'
require_relative 'treasure_trove'

module StudioGame
    class Game
        attr_reader :title
        def initialize(title)
            @title = title.capitalize
            @players = []
        end

        def add_player(player)
            @players << player
        end

        def play(rounds)
            puts "\nThere are #{@players.size} in #{@title}."

            @players.each do |player|
                puts player
            end

            treasures = TreasureTrove::TREASURES

            puts "\nThere are #{treasures.size} treasures to be found:"

            treasures.each do |t|
                puts "A #{t.name} is worth #{t.points} points"
            end

            1.upto(rounds) do |r|
                if block_given?
                    break if yield
                end
                
                puts "\n"
                puts "Round #{r}".center(20, '-')
                @players.each do |player|
                    GameTurn.take_turn(player)
                    puts player
                end
            end

            puts "Game Over".center(50, '*')
        end

        def print_name_and_health(player)
            puts "#{player.name} (#{player.health})\n"
        end

        def print_stats
            strong_players, wimpy_players = @players.partition {|p| p.strong?}
            puts "\n#{@title} Statistics:\n"
            puts "\n#{strong_players.size} strong players:"
            
            strong_players.each do |p|
                print_name_and_health(p)
            end

            puts "\n#{wimpy_players.size} wimpy players:"
            
            wimpy_players.each do |p|
                print_name_and_health(p)
            end

            puts "\n#{@title} High Scores:"
            
            @players.sort.each do |p|
                puts high_score_entry(p)
            end

            puts "\n"
            @players.sort.each do |player|
                puts "#{player.name}'s point totals:"
                player.each_found_treasure do |treasure|
                    puts "#{treasure.points} total #{treasure.name} points"
                end
                puts "#{player.points} grand total points\n\n"
            end

            puts "#{total_points} total points from treasures found"

        end

        def total_points
            @players.reduce(0) {|sum, player| sum + player.points}
        end

        def load_players(filename)
            File.readlines(filename).each do |line|
                add_player(Player.from_csv(line))
            end
        end

        def save_high_scores(high_score_filename="high_scores.txt")
            File.open(high_score_filename, "w") do |file|
                file.puts "#{@title} High Scores:"
                @players.sort.each do |p|
                    file.puts high_score_entry(p)
                end
            end
        end

        def high_score_entry(player)
            "#{player.name}".ljust(20, '.') + "#{player.score}\n"
        end

    end
end